package org.b2b.nutch.indexer;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import org.apache.hadoop.io.UTF8;

/**
 * @author Ernesto De Santis (desantisernesto@yahoo.com.ar)
 * 
 */
public class CategoryResolver {

	private static CategoryResolver instance;

	private static String propertiesPath; // = "./plugins/index-url-category/rules.properties";

	private Properties rules;

	private CategoryResolver() {

	}

	public static CategoryResolver getInstance() {
		if (instance == null) {
			instance = new CategoryResolver();
		}
		return instance;
	}

	public String resolveCategory(UTF8 utf8) throws IOException {
		return resolveCategory(utf8, getRules());
	}

	public String resolveCategory(UTF8 utf8, Properties rules)
			throws IOException {
		if (utf8 == null) {
			return null;
		}
		return resolveCategory(utf8.toString(), rules);
	}

	public String resolveCategory(String url) throws IOException {
		return resolveCategory(url, getRules());
	}
	
	public String resolveCategory(String url, Properties rules)
			throws IOException {
		if (rules == null) {
			throw new IOException("rules properties can not be null");
		}
		if (url == null || url.equals("")) {
			return null;
		}
		String category = null;

		Iterator itRules = rules.keySet().iterator();
		while (itRules.hasNext() && category == null) {
			String rule = (String) itRules.next();

			if (url.matches(rule)) {
				category = rules.getProperty(rule);
			}
		}

		return category;
	}

	private Properties getRules() throws IOException {
		if (rules == null) {
			File rulesFile = new File(propertiesPath);
			System.out.println("url category rules path: " + rulesFile.getAbsolutePath());
			
			rules = new Properties();
			InputStream is = new FileInputStream(rulesFile);
			rules.load(is);
			System.out.println("url category rules loaded... OK");
		}
		return rules;
	}

	public static void setPropertiesPath(String propertiesPath) {
		CategoryResolver.propertiesPath = propertiesPath;
	}
	
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		String url;
		String propertiesPath;
		
		if(args.length >= 1){
			url = args[0];
		} else {
			System.out.println("use: CategoryResolver url [propertiesPath]");
			return;
		}
		if(args.length >= 2){
			propertiesPath = args[1];
		} else {
			propertiesPath = "rules.properties";
		}
		
		File propertiesFile = new File(propertiesPath);
		if(!propertiesFile.exists()){
			System.out.println("Can't find rules file: " + propertiesFile.getAbsolutePath());
			return;
		}
		
		CategoryResolver.setPropertiesPath(propertiesPath);
		CategoryResolver resolver = CategoryResolver.getInstance();
		String category = resolver.resolveCategory(url);
		if(category == null){
			System.out.println("No category match for " + url) ;
		} else {
			System.out.println("category: " + category + " for " + url) ;
		}
		
		return;
	}

}
