package org.b2b.nutch.indexer;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.UTF8;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.nutch.crawl.CrawlDatum;
import org.apache.nutch.crawl.Inlinks;
import org.apache.nutch.indexer.IndexingException;
import org.apache.nutch.indexer.IndexingFilter;
import org.apache.nutch.parse.Parse;
import org.apache.nutch.plugin.Extension;
import org.apache.nutch.plugin.ExtensionPoint;
import org.apache.nutch.plugin.PluginRepository;

/**
 * <p>Url Category IndexingFilter</p>
 * 
 * @author Ernesto De Santis (desantisernesto@yahoo.com.ar)
 * 
 */
public class UrlCategoryIndexFilter implements IndexingFilter, I_UrlCategory {

	private Configuration config;

	public Configuration getConf() {
		return config;
	}

	public void setConf(Configuration arg0) {
		config = arg0;

		ExtensionPoint ep = PluginRepository.get(config).getExtensionPoint(
				"org.apache.nutch.indexer.IndexingFilter");
		if (ep != null) {
			Extension[] extensions = ep.getExtensions();
			if (extensions != null) {
				for (int i = 0; i < extensions.length; i++){
					String rulesPath = extensions[i].getAttribute("rules-path"); 
					if( rulesPath != null){
						CategoryResolver.setPropertiesPath(rulesPath );
					}
				}
			}
		}
	}

	/**
	 * Add a category lucene field to the document.
	 *  
	 */
	public Document filter(Document doc, Parse parse, UTF8 url,
			CrawlDatum datum, Inlinks inLinks) throws IndexingException {

		String category;
		try {
			category = CategoryResolver.getInstance().resolveCategory(url);
			if (category != null && !"".equals(category)) {
				doc.add(new Field(FIELD_NAME, category, Field.Store.NO,
						Field.Index.UN_TOKENIZED));
				System.out.println("category " + category + " for: "+ url);
			} else {
				System.out.println("category WARNING: url without category: " + url);
			}
			return doc;
		} catch (IOException e) {
			System.out.println("FAIL: " + e);
			throw new IndexingException(e);
			/*
			 * } catch (Throwable t){ System.out.println("FALLO " + t); throw
			 * new IndexingException(t);
			 */
		}
	}

}
